@@ -1,12 +1,16 @@
 /* source: dalan.c */
-/* Copyright Gerhard Rieger 2001-2008 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* idea of a low level data description language. currently only a most
    primitive subset exists. */
 
+#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
+#if HAVE_STDBOOL_H
+#include <stdbool.h>
+#endif
 #include <ctype.h>
 #include "dalan.h"
 
@@ -68,6 +72,7 @@ void dalan_init(void) {
 
 /* read data description from line, write result to data; do not write
    so much data that *p exceeds n !
+   p must be initialized to 0.
    return 0 on success,
    -1 if the data was cut due to n limit,
    1 if a syntax error occurred
