@@ -1,5 +1,5 @@
 /* source: filan.c */
-/* Copyright Gerhard Rieger 2001-2008 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* the subroutine filan makes a "FILe descriptor ANalysis". It checks the
@@ -215,7 +215,6 @@ int filan_stat(
 #endif /* !HAVE_STAT64 */
 	       , int statfd, int dynfd, FILE *outfile) {
    char stdevstr[8];
-   int result;
 
    /* print header */
    if (!headprinted) {
@@ -263,9 +262,9 @@ int filan_stat(
       headprinted = 1;
    }
    if (filan_rawoutput) {
-      snprintf(stdevstr, 8, F_st_dev, buf->st_dev);
+      snprintf(stdevstr, 8, F_dev, buf->st_dev);
    } else {
-      snprintf(stdevstr, 8, "%hu,%hu", (unsigned short)buf->st_dev>>8, (unsigned short)buf->st_dev&0xff);
+      snprintf(stdevstr, 8, "%hu,%hu", (unsigned short)(buf->st_dev>>8), (unsigned short)(buf->st_dev&0xff));
    }
    fprintf(outfile, "%4d: %s\t%s\t"
 #if HAVE_STAT64
@@ -300,7 +299,7 @@ int filan_stat(
 	   buf->st_mode, buf->st_nlink, buf->st_uid,
 	   buf->st_gid,
 #if HAVE_ST_RDEV
-	   (unsigned short)buf->st_rdev>>8, (unsigned short)buf->st_rdev&0xff,
+	   (unsigned short)(buf->st_rdev>>8), (unsigned short)(buf->st_rdev&0xff),
 #endif
 	   buf->st_size
 #if HAVE_ST_BLKSIZE
@@ -374,7 +373,7 @@ int filan_stat(
    case (S_IFIFO):	/* 1, FIFO */
       break;
    case (S_IFCHR):	/* 2, character device */
-      result = cdevan(statfd, outfile);
+      cdevan(statfd, outfile);
       break;
    case (S_IFDIR):	/* 4, directory */
       break;
@@ -387,7 +386,7 @@ int filan_stat(
 #ifdef S_IFSOCK
    case (S_IFSOCK): /* 12, socket */
 #if _WITH_SOCKET
-      result = sockan(statfd, outfile);
+      sockan(statfd, outfile);
 #else
       Warn("SOCKET support not compiled in");
       return -1;
@@ -490,7 +489,10 @@ int cdevan(int fd, FILE *outfile) {
 	 return -1;
       }
       fprintf(outfile, " \tIFLAGS=%08x OFLAGS=%08x CFLAGS=%08x LFLAGS=%08x",
-	      termarg.c_iflag, termarg.c_oflag, termarg.c_cflag, termarg.c_lflag);
+	      (unsigned int)termarg.c_iflag,
+	      (unsigned int)termarg.c_oflag,
+	      (unsigned int)termarg.c_cflag,
+	      (unsigned int)termarg.c_lflag);
 
       /* and the control characters */
       if (filan_rawoutput) {
@@ -597,7 +599,10 @@ int sockan(int fd, FILE *outfile) {
       {SO_DETACH_FILTER, "DETACH_FILTER"},
 #endif
       {0, NULL} } ;
-   char optval[FILAN_OPTLEN];
+   union {
+      char c[FILAN_OPTLEN];
+      int  i[FILAN_OPTLEN/sizeof(int)];
+   } optval;
    const struct sockopt *optname;
    union sockaddr_union sockname, peername;	/* the longest I know of */
    socklen_t namelen;
@@ -607,33 +612,33 @@ int sockan(int fd, FILE *outfile) {
 #endif
 
    optlen = FILAN_OPTLEN;
-   result = Getsockopt(fd, SOL_SOCKET, SO_TYPE, optval, &optlen);
+   result = Getsockopt(fd, SOL_SOCKET, SO_TYPE, optval.c, &optlen);
    if (result < 0) {
       Debug4("getsockopt(%d, SOL_SOCKET, SO_TYPE, %p, {"F_socklen"}): %s",
-	     fd, optval, optlen, strerror(errno));
+	     fd, optval.c, optlen, strerror(errno));
    } else {
-      Debug3("fd %d: socket of type %d (\"%s\")", fd, *(int *)optval,
-	  socktypes[*(int *)optval]);
+      Debug3("fd %d: socket of type %d (\"%s\")", fd, *optval.i,
+	  socktypes[*optval.i]);
    }
 
    optname = sockopts; while (optname->so) {
       optlen = FILAN_OPTLEN;
       result =
-	 Getsockopt(fd, SOL_SOCKET, optname->so, (void *)optval, &optlen);
+	 Getsockopt(fd, SOL_SOCKET, optname->so, (void *)optval.c, &optlen);
       if (result < 0) {
 	 Debug5("getsockopt(%d, SOL_SOCKET, %d, %p, {"F_socklen"}): %s",
-		fd, optname->so, optval, optlen, strerror(errno));
+		fd, optname->so, optval.c, optlen, strerror(errno));
 	 fputc('\t', outfile);
       } else if (optlen == sizeof(int)) {
 	 Debug2("getsockopt(,,, {%d}, %d)",
-	     *(int *)optval, optlen);
-	 /*Info2("%s: %d", optname->name, *(int *)optval);*/
-	 fprintf(outfile, "%s=%d\t", optname->name, *(int *)optval);
+	     *optval.i, optlen);
+	 /*Info2("%s: %d", optname->name, optval.i);*/
+	 fprintf(outfile, "%s=%d\t", optname->name, *optval.i);
       } else {
 	 Debug3("getsockopt(,,, {%d,%d}, %d)",
-	     ((int *)optval)[0], ((int *)optval)[1], optlen);
+	     optval.i[0], optval.i[1], optlen);
 	 fprintf(outfile, "%s={%d,%d}\t", optname->name,
-		 ((int *)optval)[0], ((int *)optval)[1]);
+		 optval.i[0], optval.i[1]);
       }
       ++optname;
    }
@@ -888,16 +893,19 @@ int tcpan(int fd, FILE *outfile) {
 #if _WITH_SOCKET
 int sockoptan(int fd, const struct sockopt *optname, int socklay, FILE *outfile) {
 #define FILAN_OPTLEN 256
-   char optval[FILAN_OPTLEN];
+   union {
+      char c[FILAN_OPTLEN];
+      int  i[FILAN_OPTLEN/sizeof(int)];
+   } optval;
    socklen_t optlen;
    int result;
 
    optlen = FILAN_OPTLEN;
    result =
-      Getsockopt(fd, socklay, optname->so, (void *)optval, &optlen);
+      Getsockopt(fd, socklay, optname->so, (void *)optval.c, &optlen);
    if (result < 0) {
       Debug6("getsockopt(%d, %d, %d, %p, {"F_socklen"}): %s",
-	     fd, socklay, optname->so, optval, optlen, strerror(errno));
+	     fd, socklay, optname->so, optval.c, optlen, strerror(errno));
       fputc('\t', outfile);
       return -1;
    } else if (optlen == 0) {
@@ -905,13 +913,13 @@ int sockoptan(int fd, const struct sockopt *optname, int socklay, FILE *outfile)
       fprintf(outfile, "%s=\"\"\t", optname->name);
    } else if (optlen == sizeof(int)) {
       Debug2("getsockopt(,,, {%d}, %d)",
-	     *(int *)optval, optlen);
-      fprintf(outfile, "%s=%d\t", optname->name, *(int *)optval);
+	     *optval.i, optlen);
+      fprintf(outfile, "%s=%d\t", optname->name, *optval.i);
    } else {
       char outbuf[FILAN_OPTLEN*9+128], *cp = outbuf;
       int i;
       for (i = 0; i < optlen/sizeof(unsigned int); ++i) {
-	 cp += sprintf(cp, "%08x ", ((unsigned int *)optval)[i]);
+	 cp += sprintf(cp, "%08x ", (unsigned int)optval.i[i]);
       }
       *--cp = '\0';	/* delete trailing space */
       Debug2("getsockopt(,,, {%s}, %d)", outbuf, optlen);
