@@ -1,5 +1,5 @@
 /* source: sslcls.c */
-/* Copyright Gerhard Rieger 2001-2007 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* explicit system call and C library trace function, for those who miss strace
@@ -35,71 +35,131 @@ int sycSSL_library_init(void) {
    return result;
 }
 
-SSL_METHOD *sycSSLv2_client_method(void) {
-   SSL_METHOD *result;
+#if HAVE_SSLv2_client_method
+const SSL_METHOD *sycSSLv2_client_method(void) {
+   const SSL_METHOD *result;
    Debug("SSLv2_client_method()");
    result = SSLv2_client_method();
    Debug1("SSLv2_client_method() -> %p", result);
    return result;
 }
+#endif
 
-SSL_METHOD *sycSSLv2_server_method(void) {
-   SSL_METHOD *result;
+#if HAVE_SSLv2_server_method
+const SSL_METHOD *sycSSLv2_server_method(void) {
+   const SSL_METHOD *result;
    Debug("SSLv2_server_method()");
    result = SSLv2_server_method();
    Debug1("SSLv2_server_method() -> %p", result);
    return result;
 }
+#endif
 
-SSL_METHOD *sycSSLv3_client_method(void) {
-   SSL_METHOD *result;
+const SSL_METHOD *sycSSLv3_client_method(void) {
+   const SSL_METHOD *result;
    Debug("SSLv3_client_method()");
    result = SSLv3_client_method();
    Debug1("SSLv3_client_method() -> %p", result);
    return result;
 }
 
-SSL_METHOD *sycSSLv3_server_method(void) {
-   SSL_METHOD *result;
+const SSL_METHOD *sycSSLv3_server_method(void) {
+   const SSL_METHOD *result;
    Debug("SSLv3_server_method()");
    result = SSLv3_server_method();
    Debug1("SSLv3_server_method() -> %p", result);
    return result;
 }
 
-SSL_METHOD *sycSSLv23_client_method(void) {
-   SSL_METHOD *result;
+const SSL_METHOD *sycSSLv23_client_method(void) {
+   const SSL_METHOD *result;
    Debug("SSLv23_client_method()");
    result = SSLv23_client_method();
    Debug1("SSLv23_client_method() -> %p", result);
    return result;
 }
 
-SSL_METHOD *sycSSLv23_server_method(void) {
-   SSL_METHOD *result;
+const SSL_METHOD *sycSSLv23_server_method(void) {
+   const SSL_METHOD *result;
    Debug("SSLv23_server_method()");
    result = SSLv23_server_method();
    Debug1("SSLv23_server_method() -> %p", result);
    return result;
 }
 
-SSL_METHOD *sycTLSv1_client_method(void) {
-   SSL_METHOD *result;
+const SSL_METHOD *sycTLSv1_client_method(void) {
+   const SSL_METHOD *result;
    Debug("TLSv1_client_method()");
    result = TLSv1_client_method();
    Debug1("TLSv1_client_method() -> %p", result);
    return result;
 }
 
-SSL_METHOD *sycTLSv1_server_method(void) {
-   SSL_METHOD *result;
+const SSL_METHOD *sycTLSv1_server_method(void) {
+   const SSL_METHOD *result;
    Debug("TLSv1_server_method()");
    result = TLSv1_server_method();
    Debug1("TLSv1_server_method() -> %p", result);
    return result;
 }
 
-SSL_CTX *sycSSL_CTX_new(SSL_METHOD *method) {
+#if HAVE_TLSv1_1_client_method
+const SSL_METHOD *sycTLSv1_1_client_method(void) {
+   const SSL_METHOD *result;
+   Debug("TLSv1_1_client_method()");
+   result = TLSv1_1_client_method();
+   Debug1("TLSv1_1_client_method() -> %p", result);
+   return result;
+}
+#endif
+
+#if HAVE_TLSv1_1_server_method
+const SSL_METHOD *sycTLSv1_1_server_method(void) {
+   const SSL_METHOD *result;
+   Debug("TLSv1_1_server_method()");
+   result = TLSv1_1_server_method();
+   Debug1("TLSv1_1_server_method() -> %p", result);
+   return result;
+}
+#endif
+
+#if HAVE_TLSv1_2_client_method
+const SSL_METHOD *sycTLSv1_2_client_method(void) {
+   const SSL_METHOD *result;
+   Debug("TLSv1_2_client_method()");
+   result = TLSv1_2_client_method();
+   Debug1("TLSv1_2_client_method() -> %p", result);
+   return result;
+}
+#endif
+
+#if HAVE_TLSv1_2_server_method
+const SSL_METHOD *sycTLSv1_2_server_method(void) {
+   const SSL_METHOD *result;
+   Debug("TLSv1_2_server_method()");
+   result = TLSv1_2_server_method();
+   Debug1("TLSv1_2_server_method() -> %p", result);
+   return result;
+}
+#endif
+
+const SSL_METHOD *sycDTLSv1_client_method(void) {
+   const SSL_METHOD *result;
+   Debug("DTLSv1_client_method()");
+   result = DTLSv1_client_method();
+   Debug1("DTLSv1_client_method() -> %p", result);
+   return result;
+}
+
+const SSL_METHOD *sycDTLSv1_server_method(void) {
+   const SSL_METHOD *result;
+   Debug("DTLSv1_server_method()");
+   result = DTLSv1_server_method();
+   Debug1("DTLSv1_server_method() -> %p", result);
+   return result;
+}
+
+SSL_CTX *sycSSL_CTX_new(const SSL_METHOD *method) {
    SSL_CTX *result;
    Debug1("SSL_CTX_new(%p)", method);
    result = SSL_CTX_new(method);
@@ -119,8 +179,8 @@ int sycSSL_CTX_load_verify_locations(SSL_CTX *ctx, const char *CAfile,
 				     const char *CApath) {
    int result;
    Debug7("SSL_CTX_load_verify_locations(%p, %s%s%s, %s%s%s)", ctx,
-	  CAfile?"\"":"", CAfile?CAfile:NULL, CAfile?"\"":"",
-	  CApath?"\"":"", CApath?CApath:NULL, CApath?"\"":"");
+	  CAfile?"\"":"", CAfile?CAfile:"", CAfile?"\"":"",
+	  CApath?"\"":"", CApath?CApath:"", CApath?"\"":"");
    result = SSL_CTX_load_verify_locations(ctx, CAfile, CApath);
    Debug1("SSL_CTX_load_verify_locations() -> %d", result);
    return result;
@@ -315,4 +375,42 @@ int sycFIPS_mode_set(int onoff) {
 }
 #endif /* WITH_FIPS */
 
+#if OPENSSL_VERSION_NUMBER >= 0x00908000L
+const COMP_METHOD *sycSSL_get_current_compression(SSL *ssl) {
+   const COMP_METHOD *result;
+   Debug1("SSL_get_current_compression(%p)", ssl);
+   result = SSL_get_current_compression(ssl);
+   if (result) {
+      Debug1("SSL_get_current_compression() -> %p", result);
+   } else {
+      Debug("SSL_get_current_compression() -> NULL");
+   }
+   return result;
+}
+
+const COMP_METHOD *sycSSL_get_current_expansion(SSL *ssl) {
+   const COMP_METHOD *result;
+   Debug1("SSL_get_current_expansion(%p)", ssl);
+   result = SSL_get_current_expansion(ssl);
+   if (result) {
+      Debug1("SSL_get_current_expansion() -> %p", result);
+   } else {
+      Debug("SSL_get_current_expansion() -> NULL");
+   }
+   return result;
+}
+
+const char *sycSSL_COMP_get_name(const COMP_METHOD *comp) {
+   const char *result;
+   Debug1("SSL_COMP_get_name(%p)", comp);
+   result = SSL_COMP_get_name(comp);
+   if (result) {
+      Debug1("SSL_COMP_get_name() -> \"%s\"", result);
+   } else {
+      Debug("SSL_COMP_get_name() -> NULL");
+   }
+   return result;
+}
+#endif
+
 #endif /* WITH_SYCLS && WITH_OPENSSL */
