@@ -1,5 +1,5 @@
 /* source: sslcls.h */
-/* Copyright Gerhard Rieger 2001-2007 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __sslcls_h_included
@@ -10,15 +10,21 @@
 
 void sycSSL_load_error_strings(void);
 int sycSSL_library_init(void);
-SSL_METHOD *sycSSLv2_client_method(void);
-SSL_METHOD *sycSSLv2_server_method(void);
-SSL_METHOD *sycSSLv3_client_method(void);
-SSL_METHOD *sycSSLv3_server_method(void);
-SSL_METHOD *sycSSLv23_client_method(void);
-SSL_METHOD *sycSSLv23_server_method(void);
-SSL_METHOD *sycTLSv1_client_method(void);
-SSL_METHOD *sycTLSv1_server_method(void);
-SSL_CTX *sycSSL_CTX_new(SSL_METHOD *method);
+const SSL_METHOD *sycSSLv2_client_method(void);
+const SSL_METHOD *sycSSLv2_server_method(void);
+const SSL_METHOD *sycSSLv3_client_method(void);
+const SSL_METHOD *sycSSLv3_server_method(void);
+const SSL_METHOD *sycSSLv23_client_method(void);
+const SSL_METHOD *sycSSLv23_server_method(void);
+const SSL_METHOD *sycTLSv1_client_method(void);
+const SSL_METHOD *sycTLSv1_server_method(void);
+const SSL_METHOD *sycTLSv1_1_client_method(void);
+const SSL_METHOD *sycTLSv1_1_server_method(void);
+const SSL_METHOD *sycTLSv1_2_client_method(void);
+const SSL_METHOD *sycTLSv1_2_server_method(void);
+const SSL_METHOD *sycDTLSv1_client_method(void);
+const SSL_METHOD *sycDTLSv1_server_method(void);
+SSL_CTX *sycSSL_CTX_new(const SSL_METHOD *method);
 SSL *sycSSL_new(SSL_CTX *ctx);
 int sycSSL_CTX_load_verify_locations(SSL_CTX *ctx, const char *CAfile,
 				     const char *CApath);
@@ -49,6 +55,12 @@ BIO *sycBIO_new_file(const char *filename, const char *mode);
 
 int sycFIPS_mode_set(int onoff);
 
+#if OPENSSL_VERSION_NUMBER >= 0x00908000L
+const COMP_METHOD *sycSSL_get_current_compression(SSL *ssl);
+const COMP_METHOD *sycSSL_get_current_expansion(SSL *ssl);
+const char *sycSSL_COMP_get_name(const COMP_METHOD *comp);
+#endif
+
 #endif /* WITH_OPENSSL */
 
 #else /* !WITH_SYCLS */
@@ -92,6 +104,10 @@ int sycFIPS_mode_set(int onoff);
 
 #define sycBIO_new_file(f,m) BIO_new_file(f,m)
 
+#define sycSSL_get_current_compression(s) SSL_get_current_compression(s)
+#define sycSSL_get_current_expansion(s) SSL_get_current_expansion(s)
+#define sycSSL_COMP_get_name(c) SSL_COMP_get_name(c)
+
 #endif /* WITH_OPENSSL */
 
 #define sycFIPS_mode_set(o) FIPS_mode_set(o)
