@@ -1,5 +1,5 @@
 /* source: sycls.h */
-/* Copyright Gerhard Rieger 2001-2007 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __sycls_h_included
@@ -37,6 +37,7 @@ int Setgid(gid_t gid);
 int Initgroups(const char *user, gid_t group);
 int Getgroups(int size, gid_t list[]);
 int Setgroups(size_t size, const gid_t *list);
+int Getgrouplist(const char *user, gid_t group, gid_t *groups, int *ngroups);
 int Chdir(const char *path);
 int Chroot(const char *path);
 int Gettimeofday(struct timeval *tv, struct timezone *tz);
@@ -64,6 +65,7 @@ int Ftruncate64(int fd, off64_t length);
 #endif /* HAVE_FTRUNCATE64 */
 int Flock(int fd, int operation);
 int Ioctl(int d, int request, void *argp);
+int Ioctl_int(int d, int request, int arg);
 int Close(int fd);
 int Fchown(int fd, uid_t owner, gid_t group);
 int Fchmod(int fd, mode_t mode);
@@ -92,8 +94,8 @@ int System(const char *string);
 int Socketpair(int d, int type, int protocol, int sv[2]);
 #if _WITH_SOCKET
 int Socket(int domain, int type, int protocol);
-int Bind(int sockfd, struct sockaddr *my_addr, int addrlen);
-int Connect(int sockfd, const struct sockaddr *serv_addr, int addrlen);
+int Bind(int sockfd, struct sockaddr *my_addr, socklen_t addrlen);
+int Connect(int sockfd, const struct sockaddr *serv_addr, socklen_t addrlen);
 int Listen(int s, int backlog);
 int Accept(int s, struct sockaddr *addr, socklen_t *addrlen);
 int Getsockname(int s, struct sockaddr *name, socklen_t *namelen);
@@ -137,6 +139,8 @@ int Atexit(void (*func)(void));
 void Exit(int status);
 void Abort(void);
 int Mkstemp(char *template);
+int Setenv(const char *name, const char *value, int overwrite);
+void Unsetenv(const char *name);
 
 char *Readline(const char *prompt);
 void Using_history(void);
@@ -172,6 +176,7 @@ void Add_history(const char *string);
 #define Initgroups(u,g) initgroups(u,g)
 #define Getgroups(s,l) getgroups(s,l)
 #define Setgroups(s,l) setgroups(s,l)
+#define Getgrouplist(u,g,gs,n) getgrouplist(u,g,gs,n)
 #define Chdir(p) chdir(p)
 #define Chroot(p) chroot(p)
 #define Gettimeofday(tv,tz) gettimeofday(tv,tz)
@@ -195,6 +200,7 @@ void Add_history(const char *string);
 #define Ftruncate64(f,l) ftruncate64(f,l)
 #define Flock(f,o) flock(f,o)
 #define Ioctl(d,r,a) ioctl(d,r,a)
+#define Ioctl_int(d,r,a) ioctl(d,r,a)
 #define Close(f) close(f)
 #define Fchown(f,o,g) fchown(f,o,g)
 #define Fchmod(f,m) fchmod(f,m)
@@ -256,6 +262,8 @@ void Add_history(const char *string);
 #define Exit(s) exit(s)
 #define Abort() abort()
 #define Mkstemp(t) mkstemp(t)
+#define Setenv(n,v,o) setenv(n,v,o)
+#define Unsetenv(n) unsetenv(n)
 
 #define Readline(p) readline(p)
 #define Using_history() using_history()
