@@ -1,5 +1,5 @@
 /* source: utils.c */
-/* Copyright Gerhard Rieger 2001-2008 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* useful additions to C library */
@@ -14,7 +14,7 @@
 #include "utils.h"
 
 
-#if !HAVE_MEMRCHR
+#if !HAVE_PROTOTYPE_LIB_memrchr
 /* GNU extension, available since glibc 2.1.91 */
 void *memrchr(const void *s, int c, size_t n) {
    const unsigned char *t = ((unsigned char *)s)+n;
@@ -25,7 +25,7 @@ void *memrchr(const void *s, int c, size_t n) {
       return NULL;
    return (void *)t;
 }
-#endif /* !HAVE_MEMRCHR */
+#endif /* !HAVE_PROTOTYPE_LIB_memrchr */
 
 void *memdup(const void *src, size_t n) {
    void *dest;
@@ -67,7 +67,7 @@ const struct wordent *keyw(const struct wordent *keywds, const char *name, unsig
    return NULL;
 }
 
-/* Linux: setenv(), AIX: putenv() */
+/* Linux: setenv(), AIX (4.3?): putenv() */
 #if !HAVE_SETENV
 int setenv(const char *name, const char *value, int overwrite) {
    int result;
@@ -75,7 +75,7 @@ int setenv(const char *name, const char *value, int overwrite) {
    if (!overwrite) {
       if (getenv(name))  return 0;	/* already exists */
    }
-   if ((env = Malloc(strlen(name)+strlen(value)+2)) != NULL) {
+   if ((env = Malloc(strlen(name)+strlen(value)+2)) == NULL) {
       return -1;
    }
    sprintf(env, "%s=%s", name, value);
@@ -90,9 +90,9 @@ int setenv(const char *name, const char *value, int overwrite) {
 
 
 
-/* sanitize an "untrusted" character. output buffer must provide at least 5
+/* sanitizes an "untrusted" character. output buffer must provide at least 4
    characters space.
-   Does not append null. returns length out output (currently: max 4) */
+   Does not append \0. returns length of output (currently: max 4) */
 static size_t sanitize_char(char c, char *o, int style) {
    int hn;	/* high nibble */
    int ln;	/* low nibble */
@@ -126,10 +126,12 @@ static size_t sanitize_char(char c, char *o, int style) {
    return n;
 }
 
-/* sanitize "untrusted" text, replacing special control characters with the C
-   string version ("\x"), and replacing unprintable chars with ".".
+/* sanitizes "untrusted" text, replacing special control characters with the C
+   string version (eg."\n"), and replacing unprintable chars with hex
+   representation ("\xAB").
    text can grow to four times of input, so keep output buffer long enough!
    returns a pointer to the first untouched byte of the output buffer.
+   Output is not \0 terminated.
 */
 char *sanitize_string(const char *data,	/* input data */
 		   size_t bytes,	/* length of input data, >=0 */
@@ -159,3 +161,22 @@ char *xiosubstr(char *scratch, const char *str, size_t from, size_t len) {
    return scratch0;
 }
       
+
+/* since version 1.7.2.4 socat supports C-99 behaviour of snprintf but still
+   can handle the old glibc case with -1 return on truncation.
+   Do not rely on exact return value in case of truncation
+ */
+int xio_snprintf(char *str, size_t size, const char *format, ...) {
+   va_list ap;
+   int result;
+
+   va_start(ap, format);
+   result = vsnprintf(str, size, format, ap);
+#if ! HAVE_C99_SNPRINTF
+   if (result < 0) {
+      result = size+63;	/* indicate truncation with just some guess */
+   }
+#endif /* !HAVE_C99_SNPRINTF */
+   va_end(ap);
+   return result;
+}
