@@ -1,5 +1,5 @@
 /* source: xio-ip.c */
-/* Copyright Gerhard Rieger 2001-2008 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for IP related functions */
@@ -75,15 +75,15 @@ const struct optdesc opt_ip_recvif = { "ip-recvif", "recvdstaddrif",OPT_IP_RECVI
 #endif
 
 #if HAVE_RESOLV_H
-const struct optdesc opt_res_debug    = { "res-debug",    NULL,       OPT_RES_DEBUG,    GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, (size_t)&((xiosingle_t *)0)->para.socket.ip.res_opts, sizeof(unsigned long), RES_DEBUG };
-const struct optdesc opt_res_aaonly   = { "res-aaonly",   "aaonly",   OPT_RES_AAONLY,   GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, (size_t)&((xiosingle_t *)0)->para.socket.ip.res_opts, sizeof(unsigned long), RES_AAONLY };
-const struct optdesc opt_res_usevc    = { "res-usevc",    "usevc",    OPT_RES_USEVC,    GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, (size_t)&((xiosingle_t *)0)->para.socket.ip.res_opts, sizeof(unsigned long), RES_USEVC };
-const struct optdesc opt_res_primary  = { "res-primary",  "primary",  OPT_RES_PRIMARY,  GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, (size_t)&((xiosingle_t *)0)->para.socket.ip.res_opts, sizeof(unsigned long), RES_PRIMARY };
-const struct optdesc opt_res_igntc    = { "res-igntc",    "igntc",    OPT_RES_IGNTC,    GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, (size_t)&((xiosingle_t *)0)->para.socket.ip.res_opts, sizeof(unsigned long), RES_IGNTC };
-const struct optdesc opt_res_recurse  = { "res-recurse",  "recurse",  OPT_RES_RECURSE,  GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, (size_t)&((xiosingle_t *)0)->para.socket.ip.res_opts, sizeof(unsigned long), RES_RECURSE };
-const struct optdesc opt_res_defnames = { "res-defnames", "defnames", OPT_RES_DEFNAMES, GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, (size_t)&((xiosingle_t *)0)->para.socket.ip.res_opts, sizeof(unsigned long), RES_DEFNAMES };
-const struct optdesc opt_res_stayopen = { "res-stayopen", "stayopen", OPT_RES_STAYOPEN, GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, (size_t)&((xiosingle_t *)0)->para.socket.ip.res_opts, sizeof(unsigned long), RES_STAYOPEN };
-const struct optdesc opt_res_dnsrch   = { "res-dnsrch",   "dnsrch",   OPT_RES_DNSRCH,   GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, (size_t)&((xiosingle_t *)0)->para.socket.ip.res_opts, sizeof(unsigned long), RES_DNSRCH };
+const struct optdesc opt_res_debug    = { "res-debug",    NULL,       OPT_RES_DEBUG,    GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, XIO_OFFSETOF(para.socket.ip.res_opts), XIO_SIZEOF(para.socket.ip.res_opts), RES_DEBUG };
+const struct optdesc opt_res_aaonly   = { "res-aaonly",   "aaonly",   OPT_RES_AAONLY,   GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, XIO_OFFSETOF(para.socket.ip.res_opts), XIO_SIZEOF(para.socket.ip.res_opts), RES_AAONLY };
+const struct optdesc opt_res_usevc    = { "res-usevc",    "usevc",    OPT_RES_USEVC,    GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, XIO_OFFSETOF(para.socket.ip.res_opts), XIO_SIZEOF(para.socket.ip.res_opts), RES_USEVC };
+const struct optdesc opt_res_primary  = { "res-primary",  "primary",  OPT_RES_PRIMARY,  GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, XIO_OFFSETOF(para.socket.ip.res_opts), XIO_SIZEOF(para.socket.ip.res_opts), RES_PRIMARY };
+const struct optdesc opt_res_igntc    = { "res-igntc",    "igntc",    OPT_RES_IGNTC,    GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, XIO_OFFSETOF(para.socket.ip.res_opts), XIO_SIZEOF(para.socket.ip.res_opts), RES_IGNTC };
+const struct optdesc opt_res_recurse  = { "res-recurse",  "recurse",  OPT_RES_RECURSE,  GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, XIO_OFFSETOF(para.socket.ip.res_opts), XIO_SIZEOF(para.socket.ip.res_opts), RES_RECURSE };
+const struct optdesc opt_res_defnames = { "res-defnames", "defnames", OPT_RES_DEFNAMES, GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, XIO_OFFSETOF(para.socket.ip.res_opts), XIO_SIZEOF(para.socket.ip.res_opts), RES_DEFNAMES };
+const struct optdesc opt_res_stayopen = { "res-stayopen", "stayopen", OPT_RES_STAYOPEN, GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, XIO_OFFSETOF(para.socket.ip.res_opts), XIO_SIZEOF(para.socket.ip.res_opts), RES_STAYOPEN };
+const struct optdesc opt_res_dnsrch   = { "res-dnsrch",   "dnsrch",   OPT_RES_DNSRCH,   GROUP_SOCK_IP, PH_INIT, TYPE_BOOL, OFUNC_OFFSET_MASKS, XIO_OFFSETOF(para.socket.ip.res_opts), XIO_SIZEOF(para.socket.ip.res_opts), RES_DNSRCH };
 #endif /* HAVE_RESOLV_H */
 
 #endif /* WITH_IP4 || WITH_IP6 */
@@ -129,7 +129,7 @@ int xiogetaddrinfo(const char *node, const char *service,
 #if HAVE_GETADDRINFO
    struct addrinfo hints = {0};
    struct addrinfo *res = NULL;
-#else /* HAVE_GETIPNODEBYNAME || nothing */
+#else /* HAVE_PROTOTYPE_LIB_getipnodebyname || nothing */
    struct hostent *host;
 #endif
    int error_num;
@@ -149,6 +149,9 @@ int xiogetaddrinfo(const char *node, const char *service,
    memset(sau, 0, *socklen);
    sau->soa.sa_family = family;
 
+   if (service && service[0]=='\0') {
+      Error("empty port/service");
+   }
    /* if service is numeric we don't want to have a lookup (might take long
       with NIS), so we handle this specially */
    if (service && isdigit(service[0]&0xff)) {
@@ -199,7 +202,7 @@ int xiogetaddrinfo(const char *node, const char *service,
 #endif
 	 return STAT_NORETRY;
       }
-      strncpy(numnode, node+1, nodelen-2);
+      strncpy(numnode, node+1, nodelen-2);	/* ok */
       numnode[nodelen-2] = '\0';
       node = numnode;
 #if HAVE_GETADDRINFO
@@ -314,7 +317,7 @@ int xiogetaddrinfo(const char *node, const char *service,
       }
    }
 
-#elif HAVE_GETIPNODEBYNAME /* !HAVE_GETADDRINFO */
+#elif HAVE_PROTOTYPE_LIB_getipnodebyname /* !HAVE_GETADDRINFO */
 
    if (node != NULL) {
       /* first fallback is getipnodebyname() */
@@ -362,7 +365,7 @@ int xiogetaddrinfo(const char *node, const char *service,
       freehostent(host);
    }
 
-#else /* !HAVE_GETIPNODEBYNAME */
+#else /* !HAVE_PROTOTYPE_LIB_getipnodebyname */
 
    if (node != NULL) {
       /* this is not a typical IP6 resolver function - but Linux
@@ -445,7 +448,17 @@ int xiogetaddrinfo(const char *node, const char *service,
 
 
 #if defined(HAVE_STRUCT_CMSGHDR) && defined(CMSG_DATA)
-/* these are valid for IPv4 and IPv6 */
+/* converts the ancillary message in *cmsg into a form useable for further
+   processing. knows the specifics of common message types.
+   these are valid for IPv4 and IPv6
+   returns the number of resulting syntax elements in *num
+   returns a sequence of \0 terminated type strings in *typbuff
+   returns a sequence of \0 terminated name strings in *nambuff
+   returns a sequence of \0 terminated value strings in *valbuff
+   the respective len parameters specify the available space in the buffers
+   returns STAT_OK on success
+   returns STAT_WARNING if a buffer was too short and data truncated.
+ */
 int xiolog_ancillary_ip(struct cmsghdr *cmsg, int *num,
 			char *typbuff, int typlen,
 			char *nambuff, int namlen,
@@ -454,15 +467,18 @@ int xiolog_ancillary_ip(struct cmsghdr *cmsg, int *num,
    const char *cmsgtype, *cmsgname = NULL, *cmsgenvn = NULL, *cmsgfmt = NULL;
    size_t msglen;
    char scratch1[16];	/* can hold an IPv4 address in ASCII */
+#if WITH_IP4 && defined(IP_PKTINFO) && HAVE_STRUCT_IN_PKTINFO
    char scratch2[16];
    char scratch3[16];
+#endif
+   int rc = 0;
 
    msglen = cmsg->cmsg_len-((char *)CMSG_DATA(cmsg)-(char *)cmsg);
    envbuff[0] = '\0';
    switch (cmsg->cmsg_type) {
    default:
       *num = 1;
-      strncpy(typbuff, "IP", typlen);
+      typbuff[0] = '\0'; strncat(typbuff, "IP", typlen-1);
       snprintf(nambuff, namlen, "type_%u", cmsg->cmsg_type);
       xiodump(CMSG_DATA(cmsg), msglen, valbuff, vallen, 0);
       return STAT_OK;
@@ -471,24 +487,31 @@ int xiolog_ancillary_ip(struct cmsghdr *cmsg, int *num,
    case IP_PKTINFO: {
       struct in_pktinfo *pktinfo = (struct in_pktinfo *)CMSG_DATA(cmsg);
       *num = 3;
-      strncpy(typbuff, "IP_PKTINFO", typlen);
+      typbuff[0] = '\0'; strncat(typbuff, "IP_PKTINFO", typlen-1);
       snprintf(nambuff, namlen, "%s%c%s%c%s", "if", '\0', "locaddr", '\0', "dstaddr");
       snprintf(envbuff, envlen, "%s%c%s%c%s", "IP_IF", '\0', 
 	       "IP_LOCADDR", '\0', "IP_DSTADDR");
       snprintf(valbuff, vallen, "%s%c%s%c%s",
 	       xiogetifname(pktinfo->ipi_ifindex, scratch1, -1), '\0',
-	       inet4addr_info(ntohl(pktinfo->ipi_spec_dst.s_addr), scratch2, sizeof(scratch2)), '\0',
-	       inet4addr_info(ntohl(pktinfo->ipi_addr.s_addr), scratch3, sizeof(scratch3)));
+#if HAVE_PKTINFO_IPI_SPEC_DST
+	       inet4addr_info(ntohl(pktinfo->ipi_spec_dst.s_addr),
+			      scratch2, sizeof(scratch2)),
+#else
+	       "",
+#endif
+	       '\0',
+	       inet4addr_info(ntohl(pktinfo->ipi_addr.s_addr),
+			      scratch3, sizeof(scratch3)));
    }
       return STAT_OK;
 #endif /* defined(IP_PKTINFO) && HAVE_STRUCT_IN_PKTINFO */
 #endif /* WITH_IP4 */
-#ifdef IP_RECVERR
+#if defined(IP_RECVERR) && HAVE_STRUCT_SOCK_EXTENDED_ERR
    case IP_RECVERR: {
       struct sock_extended_err *err =
 	 (struct sock_extended_err *)CMSG_DATA(cmsg);
       *num = 6;
-      strncpy(typbuff, "IP_RECVERR", typlen);
+      typbuff[0] = '\0'; strncat(typbuff, "IP_RECVERR", typlen-1);
       snprintf(nambuff, namlen, "%s%c%s%c%s%c%s%c%s%c%s",
 	       "errno", '\0', "origin", '\0', "type", '\0',
 	       "code", '\0', "info", '\0', "data");
@@ -501,17 +524,18 @@ int xiolog_ancillary_ip(struct cmsghdr *cmsg, int *num,
 	       err->ee_code, '\0', err->ee_info, '\0', err->ee_data);
       return STAT_OK;
    }
-#endif /* IP_RECVERR */
+#endif /* defined(IP_RECVERR) && HAVE_STRUCT_SOCK_EXTENDED_ERR */
 #ifdef IP_RECVIF
    case IP_RECVIF: {
       /* spec in FreeBSD: /usr/include/net/if_dl.h */
       struct sockaddr_dl *sadl = (struct sockaddr_dl *)CMSG_DATA(cmsg);
       *num = 1;
-      strncpy(typbuff, "IP_RECVIF", typlen);
-      strncpy(nambuff, "if", namlen);
-      strncpy(envbuff, "IP_IF", envlen);
-      strncpy(valbuff,
-	      xiosubstr(scratch1, sadl->sdl_data, 0, sadl->sdl_nlen), vallen);
+      typbuff[0] = '\0'; strncat(typbuff, "IP_RECVIF", typlen-1);
+      nambuff[0] = '\0'; strncat(nambuff, "if", namlen-1);
+      envbuff[0] = '\0'; strncat(envbuff, "IP_IF", envlen-1);
+      valbuff[0] = '\0';
+      strncat(valbuff,
+	      xiosubstr(scratch1, sadl->sdl_data, 0, sadl->sdl_nlen), vallen-1);
       return STAT_OK;
    }
 #endif /* defined(IP_RECVIF) */
@@ -519,9 +543,9 @@ int xiolog_ancillary_ip(struct cmsghdr *cmsg, int *num,
 #ifdef IP_RECVDSTADDR
    case IP_RECVDSTADDR:
       *num = 1;
-      strncpy(typbuff, "IP_RECVDSTADDR", typlen);
-      strncpy(nambuff, "dstaddr", namlen);
-      strncpy(envbuff, "IP_DSTADDR", envlen);
+      typbuff[0] = '\0'; strncat(typbuff, "IP_RECVDSTADDR", typlen-1);
+      nambuff[0] = '\0'; strncat(nambuff, "dstaddr", namlen-1);
+      envbuff[0] = '\0'; strncat(envbuff, "IP_DSTADDR", envlen-1);
       inet4addr_info(ntohl(*(uint32_t *)CMSG_DATA(cmsg)), valbuff, vallen);
       return STAT_OK;
 #endif
@@ -542,13 +566,13 @@ int xiolog_ancillary_ip(struct cmsghdr *cmsg, int *num,
    /* when we come here we provide a single parameter
       with type in cmsgtype, name in cmsgname, printf format in cmsgfmt */
    *num = 1;
-   if (strlen(cmsgtype) >= typlen)  Fatal("buff too short");
-   strncpy(typbuff, cmsgtype, typlen);
-   if (strlen(cmsgname) >= namlen)  Fatal("buff too short");
-   strncpy(nambuff, cmsgname, namlen);
+   if (strlen(cmsgtype) >= typlen)  rc = STAT_WARNING;
+   typbuff[0] = '\0'; strncat(typbuff, cmsgtype, typlen-1);
+   if (strlen(cmsgname) >= namlen)  rc = STAT_WARNING;
+   nambuff[0] = '\0'; strncat(nambuff, cmsgname, namlen-1);
    if (cmsgenvn) {
-      if (strlen(cmsgenvn) >= envlen)  Fatal("buff too short");
-      strncpy(envbuff, cmsgenvn, envlen);
+      if (strlen(cmsgenvn) >= envlen)  rc = STAT_WARNING;
+      envbuff[0] = '\0'; strncat(envbuff, cmsgenvn, envlen-1);
    } else {
       envbuff[0] = '\0';
    }
@@ -557,7 +581,7 @@ int xiolog_ancillary_ip(struct cmsghdr *cmsg, int *num,
    } else {
       xiodump(CMSG_DATA(cmsg), msglen, valbuff, vallen, 0);
    }
-   return STAT_OK;
+   return rc;
 }
 #endif /* defined(HAVE_STRUCT_CMSGHDR) && defined(CMSG_DATA) */
 
