@@ -1,5 +1,5 @@
 /* source: xio-system.c */
-/* Copyright Gerhard Rieger 2001-2008 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of system type */
@@ -31,14 +31,19 @@ static int xioopen_system(int argc, const char *argv[], struct opt *opts,
 		) {
    int status;
    char *path = NULL;
+   int duptostderr;
    int result;
    const char *string = argv[1];
 
-   status = _xioopen_foxec(xioflags, &fd->stream, groups, &opts);
+   status = _xioopen_foxec(xioflags, &fd->stream, groups, &opts, &duptostderr);
    if (status < 0)  return status;
    if (status == 0) {	/* child */
       int numleft;
 
+      /* do not shutdown connections that belong our parent */
+      sock[0] = NULL;
+      sock[1] = NULL;
+
       if (setopt_path(opts, &path) < 0) {
 	 /* this could be dangerous, so let us abort this child... */
 	 Exit(1);
@@ -50,6 +55,11 @@ static int xioopen_system(int argc, const char *argv[], struct opt *opts,
 	 return STAT_NORETRY;
       }
 
+      /* only now redirect stderr */
+      if (duptostderr >= 0) {
+	 diag_dup();
+	 Dup2(duptostderr, 2);
+      }
       Info1("executing shell command \"%s\"", string);
       result = System(string);
       if (result != 0) {
