@@ -1,5 +1,5 @@
 /* source: xio-udp.c */
-/* Copyright Gerhard Rieger 2001-2008 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for handling UDP addresses */
@@ -170,7 +170,8 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
    while (true) {	/* we loop with fork or prohibited packets */
       /* now wait for some packet on this datagram socket, get its sender
 	 address, connect there, and return */
-      int one = 1;
+      int reuseaddr = dofork;
+      int doreuseaddr = (dofork != 0);
       char infobuff[256];
       union sockaddr_union _sockname;
       union sockaddr_union *la = &_sockname;	/* local address */
@@ -178,18 +179,23 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
       if ((fd->stream.fd = xiosocket(opts, pf, socktype, ipproto, E_ERROR)) < 0) {
 	 return STAT_RETRYLATER;
       }
+      doreuseaddr |= (retropt_int(opts, OPT_SO_REUSEADDR, &reuseaddr) >= 0);
       applyopts(fd->stream.fd, opts, PH_PASTSOCKET);
-      if (Setsockopt(fd->stream.fd, opt_so_reuseaddr.major,
-		     opt_so_reuseaddr.minor, &one, sizeof(one)) < 0) {
-	 Warn6("setsockopt(%d, %d, %d, {%d}, "F_Zd"): %s",
-	       fd->stream.fd, opt_so_reuseaddr.major,
-	       opt_so_reuseaddr.minor, one, sizeof(one), strerror(errno));
+      if (doreuseaddr) {
+	 if (Setsockopt(fd->stream.fd, opt_so_reuseaddr.major,
+			opt_so_reuseaddr.minor, &reuseaddr, sizeof(reuseaddr))
+	     < 0) {
+	    Warn6("setsockopt(%d, %d, %d, {%d}, "F_Zd"): %s",
+		  fd->stream.fd, opt_so_reuseaddr.major,
+		  opt_so_reuseaddr.minor, reuseaddr, sizeof(reuseaddr),
+		  strerror(errno));
+	 }
       }
       applyopts_cloexec(fd->stream.fd, opts);
       applyopts(fd->stream.fd, opts, PH_PREBIND);
       applyopts(fd->stream.fd, opts, PH_BIND);
       if (Bind(fd->stream.fd, &us.soa, uslen) < 0) {
-	 Error4("bind(%d, {%s}, "F_Zd"): %s", fd->stream.fd,
+	 Error4("bind(%d, {%s}, "F_socklen"): %s", fd->stream.fd,
 		sockaddr_info(&us.soa, uslen, infobuff, sizeof(infobuff)),
 		uslen, strerror(errno));
 	 return STAT_RETRYLATER;
@@ -215,7 +221,7 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
 			     &them->soa, &themlen);
       } while (result < 0 && errno == EINTR);
       if (result < 0) {
-	 Error5("recvfrom(%d, %p, 1, MSG_PEEK, {%s}, {"F_Zu"}): %s",
+	 Error5("recvfrom(%d, %p, 1, MSG_PEEK, {%s}, {"F_socklen"}): %s",
 		fd->stream.fd, buff1,
 		sockaddr_info(&them->soa, themlen, infobuff, sizeof(infobuff)),
 		themlen, strerror(errno));
@@ -251,7 +257,6 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
 	 if (Close(fd->stream.fd) < 0) {
 	    Info2("close(%d): %s", fd->stream.fd, strerror(errno));
 	 }
-	 Sleep(1);	/*! give child a chance to consume the old packet */
 
 	 continue;
       }
@@ -260,7 +265,7 @@ int xioopen_ipdgram_listen(int argc, const char *argv[], struct opt *opts,
 
    applyopts(fd->stream.fd, opts, PH_CONNECT);
    if ((result = Connect(fd->stream.fd, &them->soa, themlen)) < 0) {
-      Error4("connect(%d, {%s}, "F_Zd"): %s",
+      Error4("connect(%d, {%s}, "F_socklen"): %s",
 	     fd->stream.fd,
 	     sockaddr_info(&them->soa, themlen, infobuff, sizeof(infobuff)),
 	     themlen, strerror(errno));
@@ -457,7 +462,6 @@ int xioopen_udp_recvfrom(int argc, const char *argv[], struct opt *opts,
 		     int pf, int socktype, int ipproto) {
    union sockaddr_union us;
    socklen_t uslen = sizeof(us);
-   bool needbind = false;
    int result;
 
    if (argc != 2) {
@@ -505,7 +509,6 @@ int xioopen_udp_recvfrom(int argc, const char *argv[], struct opt *opts,
 	 case PF_INET6: us.ip6.sin6_addr = la.ip6.sin6_addr; break;
 #endif
 	 }
-	 needbind = true;
       }
    }
 
