@@ -1,5 +1,5 @@
 /* source: xio.h */
-/* Copyright Gerhard Rieger 2001-2009 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_h_included
@@ -201,6 +201,11 @@ typedef struct single {
 #endif
 	 } ip;
 #endif /* _WITH_IP4 || _WITH_IP6 */
+#if WITH_UNIX
+	 struct {
+	    bool     tight;
+	 } un;
+#endif /* WITH_UNIX */
       } socket;
 #endif /* _WITH_SOCKET */
       struct {
@@ -378,6 +383,8 @@ struct opt {
 extern const char *PIPESEP;
 extern xiofile_t *sock[XIO_MAXSOCK];
 
+extern int num_child;
+
 /* return values of xioopensingle */
 #define STAT_OK		0
 #define STAT_WARNING	1
@@ -402,10 +409,12 @@ extern int xioopenhelp(FILE *of, int level);
 
 /* must be outside function for use by childdied handler */
 extern xiofile_t *sock1, *sock2;
-extern pid_t diedunknown1;	/* child died before it is registered */
-extern pid_t diedunknown2;
-extern pid_t diedunknown3;
-extern pid_t diedunknown4;
+#define NUMUNKNOWN 4
+extern pid_t diedunknown[NUMUNKNOWN];	/* child died before it is registered */
+#define diedunknown1 (diedunknown[0])
+#define diedunknown2 (diedunknown[1])
+#define diedunknown3 (diedunknown[2])
+#define diedunknown4 (diedunknown[3])
 
 extern int xiosetsigchild(xiofile_t *xfd, int (*callback)(struct single *));
 extern int xiosetchilddied(void);
