@@ -1,5 +1,5 @@
 /* source: xioconfig.h */
-/* Copyright Gerhard Rieger 2001-2007 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xioconfig_h_included
@@ -20,6 +20,10 @@
 #  define WITH_NAMED 1
 #endif
 
+#if WITH_TERMIOS || WITH_PTY || WITH_READLINE
+#  define _WITH_TERMIOS 1
+#endif
+
 #if WITH_SOCKS4A
 #  define WITH_SOCKS4 1
 #endif
@@ -47,13 +51,13 @@
 #  endif
 #endif
 
-#if WITH_UNIX || WITH_IP4 || WITH_IP6 || WITH_SOCKS4 || WITH_RAWIP
-#  define WITH_SOCKET 1
+#if WITH_UNIX || WITH_IP4 || WITH_IP6 || WITH_SOCKS4 || WITH_RAWIP || WITH_GENERICSOCKET
+#  define _WITH_SOCKET 1
 #else
-#  undef WITH_SOCKET
+#  undef _WITH_SOCKET
 #endif
 
-#if !WITH_SOCKET
+#if !_WITH_SOCKET
 #  undef WITH_LISTEN
 #endif
 
@@ -61,7 +65,7 @@
 #  undef WITH_LIBWRAP
 #endif
 
-#if WITH_SOCKET || WITH_TUN
+#if WITH_GENERICSOCKET || WITH_TUN
 #  define _WITH_SOCKET 1
 #endif
 
@@ -82,12 +86,12 @@
 #endif
 
 
-#if HAVE_DEV_PTMX && HAVE_GRANTPT && HAVE_UNLOCKPT && HAVE_PTSNAME
+#if HAVE_DEV_PTMX && HAVE_GRANTPT && HAVE_UNLOCKPT && HAVE_PROTOTYPE_LIB_ptsname
 #else
 #  undef HAVE_DEV_PTMX
 #endif
 
-#if HAVE_DEV_PTC /* && HAVE_GRANTPT && HAVE_UNLOCKPT && HAVE_PTSNAME */
+#if HAVE_DEV_PTC /* && HAVE_GRANTPT && HAVE_UNLOCKPT && HAVE_PROTOTYPE_LIB_ptsname */
 #else
 #  undef HAVE_DEV_PTC
 #endif
@@ -116,4 +120,8 @@
    typedef unsigned int uint32_t;
 #endif
 
+#ifndef HAVE_TYPE_SA_FAMILY_T
+   typedef uint16_t sa_family_t;
+#endif
+
 #endif /* !defined(__xioconfig_h_included) */
