@@ -1,5 +1,5 @@
 /* source: xiomodes.h */
-/* Copyright Gerhard Rieger 2001-2007 */
+/* Copyright Gerhard Rieger 2001-2008 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xiomodes_h_included
@@ -15,11 +15,12 @@
 #include "xio-creat.h"
 #include "xio-gopen.h"
 #include "xio-pipe.h"
-#if WITH_SOCKET
+#if _WITH_SOCKET
 #include "xio-socket.h"
 #include "xio-listen.h"
 #include "xio-unix.h"
 #include "xio-rawip.h"
+#include "xio-interface.h"
 #include "xio-ip.h"
 #if WITH_IP4
 #include "xio-ip4.h"
@@ -28,9 +29,10 @@
 #include "xio-ipapp.h"
 #include "xio-tcp.h"
 #include "xio-udp.h"
+#include "xio-sctp.h"
 #include "xio-socks.h"
 #include "xio-proxy.h"
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
 #include "xio-progcall.h"
 #include "xio-exec.h"
 #include "xio-system.h"
@@ -41,5 +43,6 @@
 #include "xio-tcpwrap.h"
 #include "xio-ext2.h"
 #include "xio-tun.h"
+#include "xio-streams.h"
 
 #endif /* !defined(__xiomodes_h_included) */
