@@ -1,5 +1,5 @@
 /* source: xioopts.h */
-/* Copyright Gerhard Rieger 2001-2009 */
+/* Copyright Gerhard Rieger */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xioopts_h_included
@@ -432,6 +432,7 @@ enum e_optcode {
    OPT_IXON,		/* termios.c_iflag */
    OPT_LOCKFILE,
    OPT_LOWPORT,
+   OPT_MAX_CHILDREN,
 #ifdef NLDLY
 #  ifdef NL0
    OPT_NL0,		/* termios.c_oflag */
@@ -472,6 +473,10 @@ enum e_optcode {
    OPT_OPENSSL_CAPATH,
    OPT_OPENSSL_CERTIFICATE,
    OPT_OPENSSL_CIPHERLIST,
+   OPT_OPENSSL_COMMONNAME,
+#if OPENSSL_VERSION_NUMBER >= 0x00908000L
+   OPT_OPENSSL_COMPRESS,
+#endif
    OPT_OPENSSL_DHPARAM,
    OPT_OPENSSL_EGD,
    OPT_OPENSSL_FIPS,
@@ -703,7 +708,9 @@ enum e_optcode {
 #  undef ENABLE_OPTCODE
    OPT_SUBSTUSER_EARLY,
    OPT_SUBSTUSER,
+#if defined(HAVE_SETGRENT) && defined(HAVE_GETGRENT) && defined(HAVE_ENDGRENT)
    OPT_SUBSTUSER_DELAYED,
+#endif
    OPT_SYMBOLIC_LINK,	/* with pty */
 #ifdef TABDLY
 #  ifdef TAB0
@@ -778,6 +785,8 @@ enum e_optcode {
 #ifdef TCP_WINDOW_CLAMP
    OPT_TCP_WINDOW_CLAMP,	/* Linux 2.4.0 */
 #endif
+   OPT_TERMIOS_CFMAKERAW,	/* termios.cfmakeraw() */
+   OPT_TERMIOS_RAWER,
    OPT_TIOCSCTTY,
    OPT_TOSTOP,		/* termios.c_lflag */
    OPT_TUN_DEVICE,	/* tun: /dev/net/tun ... */
